<?php
/**
 * @author Troy <troytft@gmail.com>
 */
require(YII_PATH . '/cli/commands/MigrateCommand.php');

class EMigrateCommand extends MigrateCommand
{

    /**
     * Move migrations into folders
     */
    public function actionSort()
    {
        $handle = opendir($this->migrationPath);
        while (($file = readdir($handle)) !== false)
        {
            if ($file === '.' || $file === '..')
                continue;

            if (is_file($this->migrationPath . DIRECTORY_SEPARATOR . $file))
            {
                $path = $this->migrationPath . DIRECTORY_SEPARATOR . substr($file, 1, 4);
                if (!is_dir($path))
                    mkdir($path);

                rename($this->migrationPath . DIRECTORY_SEPARATOR . $file, $path . DIRECTORY_SEPARATOR . $file);
            }
        }
        closedir($handle);

        echo "All migrations sorted\n";
    }


    /**
     * New migrations
     * @return array Founded migrations
     */
    public function getNewMigrations()
    {
        $applied = array();
        foreach ($this->getMigrationHistory(-1) as $version => $time)
            $applied[$version] = true;

        $migrations = $this->_recursiveMigrationsSearch($this->migrationPath);
        foreach ($migrations as $k => $v)
            if (isset($applied[$v]))
                unset($migrations[$k]);

        sort($migrations);

        return $migrations;
    }


    /**
     * Recursive migrations search
     * @param string $path Path
     * @return array Founded migrations
     */
    private function _recursiveMigrationsSearch($path)
    {
        $migrations = array();
        $handle = opendir($path);
        while (($file = readdir($handle)) !== false)
        {
            if ($file === '.' || $file === '..')
                continue;

            $fullpath = $path . DIRECTORY_SEPARATOR . $file;
            if (is_dir($fullpath))
                $migrations += $this->_recursiveMigrationsSearch($fullpath);

            if (preg_match('/^(m\d{6}_\d{6}_.*?)\.php$/', $file, $matches) && is_file($fullpath))
                $migrations[] = $matches[1];
        }
        closedir($handle);

        return $migrations;
    }


}
